@@ -14,6 +14,8 @@ import sympy
 from cellmlmanip.rdf import create_rdf_node
 from cellmlmanip.units import UnitStore
 
+from . import parser
+
 
 logger = logging.getLogger(__name__)
 
@@ -139,7 +141,7 @@ class Model(object):
         """Return a display name for the given variable.
 
         Looks for an annotation in the ontology first (or the local name from any annotation if no ontology is
-        specified), then cmeta:id if present, or the variable's name attribute if not.
+        specified), then ``cmeta:id`` if present, or the variable's ``name`` attribute if not.
 
         Dollar symbols in the name are replaced by a double underscore.
 
@@ -153,11 +155,11 @@ class Model(object):
         return var.cmeta_id if var.cmeta_id else var.name.replace('$', '__')
 
     def is_state(self, variable):
-        """Checks if the given ``variable`` is a state variable (i.e. if it's defined by an ODE)."""
+        """Checks if ``variable`` is a state variable (i.e. if it's defined by an ODE)."""
         return variable in self._ode_definition_map
 
     def is_constant(self, variable):
-        """Determine whether the given ``variable`` is a constant.
+        """Determine whether ``variable`` is a constant.
 
         This is calculated by looking at the RHS of the defining equation and checking it has no
         variable references.
@@ -171,7 +173,7 @@ class Model(object):
 
     def get_variable_by_cmeta_id(self, cmeta_id):
         """
-        Searches the model and returns the variable with the given cmeta id.
+        Searches the model and returns the variable with the given ``cmeta id``.
 
         To get variables from e.g. an oxmeta ontology term, use :meth:`get_variable_by_ontology_term`.
 
@@ -199,13 +201,11 @@ class Model(object):
     def get_variable_by_ontology_term(self, term):
         """Searches the RDF graph for a variable annotated with the given ``term`` and returns it.
 
-        Specifically, this method searches for a unique variable annotated with
-        predicate http://biomodels.net/biology-qualifiers/is and the object
-        specified by ``term``.
+        Specifically, this method searches for a unique variable annotated with predicate
+        ``http://biomodels.net/biology-qualifiers/is`` and the object specified by ``term``.
 
-        Will raise a ``KeyError`` if no variable with the given annotation is
-        found, and a ``ValueError`` if more than one variable with the given
-        annotation is found.
+        Will raise a ``KeyError`` if no variable with the given annotation is found, and a ``ValueError`` if more than
+        one variable with the given annotation is found.
 
         :param term: anything suitable as an input to :meth:`cellmlmanip.rdf.create_rdf_node`; typically either an RDF
             node already, or a tuple ``(namespace_uri, local_name)``.
@@ -240,10 +240,11 @@ class Model(object):
     def get_rdf_value(self, subject, predicate):
         """Get the value of an RDF object connected to ``subject`` by ``predicate``.
 
+        Note: expects exactly one triple to match and the result to be a literal.
+
         :param subject: the object of the triple returned
         :param predicate: the object of the triple returned
-
-        Note: expects exactly one triple to match and the result to be a literal. Its string value is returned.
+        :returns: a string
         """
         triples = self.get_rdf_annotations(subject, predicate)
         triples = tuple(self.get_rdf_annotations(subject, predicate))
@@ -264,7 +265,7 @@ class Model(object):
         If all are ``None``, then all triples are returned.
 
         The arguments can be anything valid as input to :meth:`cellmlmanip.rdf.create_rdf_node`,
-        typically a (namespace URI, local name) pair, a string or ``None``.
+        typically a (namespace URI, local name) pair, a string, or ``None``.
         """
         subject = create_rdf_node(subject)
         predicate = create_rdf_node(predicate)
@@ -274,7 +275,7 @@ class Model(object):
     def get_ontology_terms_by_variable(self, variable, namespace_uri=None):
         """
         Returns all ontology terms linked to the given ``variable`` via the
-        http://biomodels.net/biology-qualifiers/is predicate.
+        ``http://biomodels.net/biology-qualifiers/is`` predicate.
 
         :param variable: The variable to search for (as a :class:`Variable` object).
         :param namespace_uri: An optional namespace URI. If given, only terms within the given namespace will be
@@ -293,8 +294,8 @@ class Model(object):
 
     def has_ontology_annotation(self, variable, namespace_uri=None):
         """
-        Checks that there is at least one result for
-        :meth:`Model.get_ontology_terms_by_variable` with the given arguments.
+        Checks that there is at least one result for :meth:`Model.get_ontology_terms_by_variable` with the given
+        arguments.
         """
         return len(self.get_ontology_terms_by_variable(variable, namespace_uri)) != 0
 
@@ -490,8 +491,8 @@ class Model(object):
     @property
     def graph_with_sympy_numbers(self):
         """
-        A :class:`networkx.DiGraph` containing the model equations,
-        but with numbers represented as :class:`sympy.Number` objects instead of :class:`Quantity`.
+        A :class:`networkx.DiGraph` containing the model equations, but with numbers represented as
+        :class:`sympy.Number` objects instead of :class:`Quantity` objects.
         """
         if self._graph_with_sympy_numbers is not None:
             return self._graph_with_sympy_numbers
@@ -554,8 +555,8 @@ class Model(object):
         :param initial_value: An optional initial value.
         :param public_interface: An optional public interface specifier (only required when parsing CellML).
         :param private_interface: An optional private interface specifier (only required when parsing CellML).
-        :param cmeta_id: An optional string specifying a cmeta:id
-        :raises ValueError: If a variable with that name already exists, or the given cmeta_id is already taken.
+        :param cmeta_id: An optional string specifying a cmeta id
+        :raises ValueError: If a variable with that name already exists, or the given cmeta id is already taken.
         :return: A :class:`Variable` object.
         """
         # Check for clashes
@@ -671,7 +672,7 @@ class Model(object):
 
     def create_quantity(self, value, units):
         """
-        Creates and returns a :class:`Quantity` to represent a number with units in sympy expressions.
+        Creates and returns a :class:`Quantity` to represent a number with units in Sympy expressions.
 
         Use this method rather than creating quantities directly to ensure their units are compatible with those used by
         the model, so unit conversion etc. works.
@@ -1042,6 +1043,33 @@ class Model(object):
 
         return new_variable
 
+    def remove_fixable_singularities(self, V, exclude=set()):
+        """
+        Removes removable singularities from the model equations and replaces these with a piecewise.
+
+        The process looks for equations of any of the following forms, where U is a function of V:
+        - `U / (exp(U) - 1.0)`
+        - `U / (1.0 - exp(U))`
+        - `(exp(U) - 1.0) / U`
+        - `(1.0 - exp(U)) / U`
+        It replaces these with a piecewise 1e-7 either side of U==0 drawing a stright line in the region.
+        For example `(V + 5)/(exp(V + 5) - 1)` becomes
+        `((fabs(-V - 5.0000000000000000) < fabs(-4.9999999000000000 / 2 - -5.0000001000000000 / 2))
+        ? -0.494049243462503*V - 1.4702462167574 : ((5.0 + V) / (-1.0 + exp(5.0 + V))))`
+
+        see [Johnstone, R. H. (2018). Uncertainty characterisation in action potential modelling for cardiac drug
+        safety. University of Oxford.](https://ora.ox.ac.uk/objects/uuid:0a28829c-828d-4641-bfb0-11193ef47195)
+
+        :param exclude: set of variables which will not be substituted in the evaluation.
+
+        This ensures their defining equations will remain.
+        """
+        from ._singularity_fixes import remove_fixable_singularities
+        if not isinstance(exclude, set):
+            raise TypeError('exclude is expected to be a set')
+        remove_fixable_singularities(self, V, exclude,
+                                     exp_function=parser.SIMPLE_MATHML_TO_SYMPY_CLASSES['exp'])
+
 
 class Quantity(sympy.Dummy):
     """
@@ -1091,12 +1119,12 @@ class Variable(sympy.Dummy):
     """
 
     # Sympy annoyingly overwrites __new__
-    def __new__(cls, name, *args, **kwargs):
-        return super().__new__(cls, name, real=True)
+    def __new__(cls, *args, **kwargs):
+        return super().__new__(cls, real=True)
 
     def __init__(self,
-                 name,
-                 units,
+                 name=None,
+                 units=None,
                  model=None,
                  initial_value=None,
                  public_interface=None,
