@@ -5,7 +5,7 @@ is handled by RDFLib.
 """
 import itertools
 import os
-from collections import OrderedDict, deque
+from collections import deque
 from enum import Enum
 
 import sympy
@@ -69,15 +69,13 @@ class _Component:
     def set_parent(self, parent_name):
         """Sets the parent of this component"""
         if self.parent:
-            raise ValueError('Parent of component %s already %s. Cannot set %s!' % (self.name,
-                                                                                    self.parent,
-                                                                                    parent_name))
+            raise ValueError('Parent of component %s already %s. Cannot set %s: multiple parents not allowed!' %
+                             (self.name, self.parent, parent_name))
         self.parent = parent_name
 
     def add_sibling(self, sibling_name):
         """Adds a sibling for this component"""
-        if sibling_name in self.siblings:
-            raise ValueError('Sibling component %s already added!' % sibling_name)
+        assert sibling_name not in self.siblings, 'Sibling component %s already added!' % sibling_name
         self.siblings.add(sibling_name)
 
     def add_encapsulated(self, encapsulated_name):
@@ -101,7 +99,7 @@ class Parser(object):
         self.model = None
 
         # A dictionary mapping component names to _Component objects
-        self.components = OrderedDict()
+        self.components = {}
 
     def parse(self, unit_store=None):
         """
@@ -132,9 +130,11 @@ class Parser(object):
         self._add_units(model_xml)
         self._add_rdf(model_xml)
 
-        self._add_components(model_xml)
+        component_variables = self._add_components(model_xml)
         self._add_relationships(model_xml)
-        self._add_connection(model_xml)
+
+        connected_variable_mapping = self._add_connections(model_xml)
+        self._add_maths(component_variables, connected_variable_mapping)
 
         # Canonicalise representation
         self.transform_constants()
@@ -166,7 +166,7 @@ class Parser(object):
         units_found = set(_CELLML_UNITS)
 
         # get all the units defined in the cellml model
-        definitions_to_add = OrderedDict()
+        definitions_to_add = deque()
         for units_element in units_elements:
             units_name = units_element.get('name')
             # if it's a defined base unit, we can be add immediately to the model
@@ -175,14 +175,14 @@ class Parser(object):
                 units_found.add(units_name)
             # all other units are collected (because they may depend on further user-defined units)
             else:
-                unit_elements = [dict(t.attrib) for t in units_element.getchildren()]
-                definitions_to_add[units_name] = unit_elements
+                unit_elements = [t.attrib for t in units_element.getchildren()]
+                definitions_to_add.append((units_name, unit_elements))
 
         iteration = 0
         # while we still have units to add
         while definitions_to_add:
             # get a definition from the top of the list
-            unit_name, unit_elements = definitions_to_add.popitem()
+            unit_name, unit_elements = definitions_to_add.pop()
 
             # check whether this unit is defined in terms of units that we know about
             add_now = True
@@ -190,8 +190,7 @@ class Parser(object):
                 # if defined in terms of units we don't know about
                 if unit['units'] not in units_found:
                     # defer adding this units - add it back to the end of the list
-                    definitions_to_add[unit_name] = unit_elements
-                    definitions_to_add.move_to_end(unit_name, last=False)
+                    definitions_to_add.appendleft((unit_name, unit_elements))
                     add_now = False
                     break
 
@@ -253,9 +252,12 @@ class Parser(object):
         """
         <model> <component> </model>
         :param model: an etree.Element
+
+        :return: a list of (element, variable_to_symbol) tuples with maths to be added later.
         """
         component_elements = model.findall(with_ns(XmlNs.CELLML, 'component'))
 
+        component_variables = []
         # for each component defined in the model
         for element in component_elements:
             # component are only kept in parser to resolve relationships and connections
@@ -265,8 +267,8 @@ class Parser(object):
             # process the <variable> tags in this component
             variable_to_symbol = self._add_variables(element)
 
-            # process the <math> tags in this component
-            self._add_maths(element, variable_to_symbol)
+            # to speed up parsing, we store the maths to add to perform adding later when we know how variables connect
+            component_variables.append((element, variable_to_symbol))
 
             # Raise error if component units are defined
             component_units = element.findall(with_ns(XmlNs.CELLML, 'units'))
@@ -279,6 +281,7 @@ class Parser(object):
             if reactions:
                 raise ValueError(
                     'Reactions are not supported (found in component ' + name + ').')
+        return component_variables
 
     def _add_variables(self, component_element):
         """
@@ -288,7 +291,7 @@ class Parser(object):
         variable_elements = component_element.findall(with_ns(XmlNs.CELLML, 'variable'))
 
         # we keep a {variable name: sympy symbol} lookup that we pass to the transpiler
-        variable_lookup_symbol = dict()
+        variable_lookup_symbol = {}
 
         for variable_element in variable_elements:
             attributes = dict(variable_element.attrib)
@@ -310,41 +313,51 @@ class Parser(object):
 
         return variable_lookup_symbol
 
-    def _add_maths(self, component_element, variable_to_symbol):
+    def _add_maths(self, component_variables, connected_variable_mapping):
         """
+        Add maths for all elements stored in component_variables.
+
         <model> <component> <math> </component> </model>
 
-        :param component_element: an etree.Element
-        :param variable_to_symbol: a ``Dict[str, sympy.Dummy]``
+        :param component_variables: a list of (component_element, variable_to_symbol) tuples
+                                    where ``component_element`` is an ``etree.Element`
+                                    and ``variable_to_symbol`` is a ``Dict[str, sympy.Dummy]``
+                                        with the key the variable name and the value the variable in the model
+
+        :param connected_variable_mapping: a ``Dict[str, sympy.Dummy]`` mapping a connected variable to its source.
         """
-        # get all <math> elements in the component
-        math_elements = component_element.findall(with_ns(XmlNs.MATHML, 'math'))
 
-        # nothing to do if we don't have any <math> elements
-        if not math_elements:
-            return
+        for component_element, variable_to_symbol in component_variables:
+            # get all <math> elements in the component
+            math_elements = component_element.findall(with_ns(XmlNs.MATHML, 'math'))
 
-        # Method to create symbols
-        prefix = component_element.get('name') + SYMPY_SYMBOL_DELIMITER
+            # nothing to do if we don't have any <math> elements
+            if math_elements:
+                # Method to create symbols
+                prefix = component_element.get('name') + SYMPY_SYMBOL_DELIMITER
 
-        def symbol_generator(identifer):
-            out = variable_to_symbol.get(prefix + identifer, None)
-            assert out is not None, '%s not found in symbol dict' % (prefix + identifer)
-            return out
+                def symbol_generator(identifer):
+                    symbol_name = prefix + identifer
+                    out = variable_to_symbol.get(symbol_name, None)
+                    while str(out) in connected_variable_mapping:
+                        out = connected_variable_mapping[str(out)]
+                    assert out is not None, '%s not found in symbol dict' % (prefix + identifer)
+                    return out
 
-        # reuse transpiler so dummy symbols are kept across <math> elements
-        transpiler = Transpiler(
-            symbol_generator=symbol_generator,
-            number_generator=lambda x, y: self.model.create_quantity(x, self.model.units.get_unit(y)),
-        )
+                # Set up transpiler to generate correct symbol names for connected variables in one go
+                # without the need for post-processing
+                transpiler = Transpiler(
+                    symbol_generator=symbol_generator,
+                    number_generator=lambda x, y: self.model.create_quantity(x, self.model.units.get_unit(y)),
+                )
 
-        # for each math element
-        for math_element in math_elements:
-            sympy_exprs = transpiler.parse_tree(math_element)
+                # for each math element
+                for math_element in math_elements:
+                    sympy_exprs = transpiler.parse_tree(math_element)
 
-            # add each equation from <math> to the model
-            for expr in sympy_exprs:
-                self.model.add_equation(expr)
+                    # add each equation from <math> to the model
+                    for expr in sympy_exprs:
+                        self.model.add_equation(expr)
 
     def _add_relationships(self, model: etree.Element):
         group_elements = model.findall(with_ns(XmlNs.CELLML, 'group'))
@@ -354,7 +367,8 @@ class Parser(object):
 
             # find the relationship for this <group>
             relationship_ref = group_element.findall(with_ns(XmlNs.CELLML, 'relationship_ref'))
-            assert len(relationship_ref) == 1
+            if len(relationship_ref) != 1:
+                raise ValueError("Expecting exactly 1 relationship_ref tag per group, got %s!" % len(relationship_ref))
             relationship = relationship_ref[0].attrib.get('relationship')
 
             # we only handle 'encapsulation' relationships (i.e. ignoring 'containment')
@@ -390,34 +404,34 @@ class Parser(object):
                 if component_a != component_b:
                     self.components[component_a].add_sibling(component_b)
 
-    def _add_connection(self, model):
+    def _add_connections(self, model):
         """
         :param model: an etree.Element
+
+        :return: dict mapping a connected variable to its source variable.
         """
+        connected_variable_mapping = {}
         connection_elements = model.findall(with_ns(XmlNs.CELLML, 'connection'))
 
         # a list to collect the (source, target) connection tuples
-        connect_from_to = []
+        connections_to_process = deque()
 
         # for each connection in the model
         for connection in connection_elements:
-            # first child is <map_component>
-            child_0 = connection[0]
-            assert child_0.tag == with_ns(XmlNs.CELLML, 'map_components')
-            comp_1, comp_2 = (child_0.attrib.get('component_1'),
-                              child_0.attrib.get('component_2'))
+            # Should have one map_components and at least one map_variables. RELAXNGV makes use of this
+            map_components = connection.find(with_ns(XmlNs.CELLML, 'map_components'))
+            comp_1, comp_2 = (map_components.attrib.get('component_1'),
+                              map_components.attrib.get('component_2'))
 
-            # the remaining children are <map_variables> tags
-            for child in connection[1:]:
-                assert child.tag == with_ns(XmlNs.CELLML, 'map_variables')
-                connect_from_to.append(
+            # go through all tags
+            for child in connection.findall(with_ns(XmlNs.CELLML, 'map_variables')):
+                connections_to_process.append(
                     self._determine_connection_direction(comp_1, child.attrib.get('variable_1'),
                                                          comp_2, child.attrib.get('variable_2'))
                 )
 
         # we add the connection to the model by first connecting
         # those variables we know are source variables
-        connections_to_process = deque(connect_from_to)
 
         # keep processing the list of connections until we've done them all
         unchanged_loop_count = 0
@@ -427,15 +441,32 @@ class Parser(object):
 
             # if we did not successfully connect this variable (because, e.g., we don't know the
             # source of *this* source)
-            if not self.model.connect_variables(*connection):
+            source, target = connection
+            # Target should not already be assigned to
+            if target.assigned_to:
+                raise ValueError('Target already assigned to %s before assignment to %s' %
+                                 (target.assigned_to, source.assigned_to))
+
+            if not source.assigned_to:
                 # add it back to the list
                 connections_to_process.append(connection)
                 unchanged_loop_count += 1
             else:
+                connected_variable_mapping[target.name] = source  # Store connection for later when maths is added
+                # Check whether we need a unit conversion
+                cf = self.model.units.get_conversion_factor(from_unit=source.units, to_unit=target.units)
+                if cf == 1:
+                    target.assigned_to = source.assigned_to  # Direct substitution is possible
+                    if target.cmeta_id is not None:  # In case annotation is on target instead of source
+                        self.model.transfer_cmeta_id(source=target, target=source)
+                else:
+                    cf_quant = self.model.create_quantity(cf, target.units / source.units)  # conversion factor quant
+                    self.model.add_equation(sympy.Eq(target, source.assigned_to * cf_quant))  # Add connecting equation
+                    target.assigned_to = target  # The assigned variable for this variable is itself
                 unchanged_loop_count = 0
 
-            if unchanged_loop_count > len(connections_to_process):
-                raise ValueError('Unable to add connections to the model')
+            assert unchanged_loop_count <= len(connections_to_process), 'Unable to add connections to the model'
+        return connected_variable_mapping
 
     def _determine_connection_direction(self, comp_1, var_1, comp_2, var_2):
         """Takes a CellML connection and attempts to resolve the connect by assigning the target
@@ -469,10 +500,12 @@ class Parser(object):
         # if the components are siblings (either same parent or top-level)
         if _are_siblings(comp_1, comp_2):
             # they are both connected on their public_interface
-            if variable_1.public_interface == 'out' and variable_2.public_interface == 'in':
-                return variable_1.name, variable_2.name
-            elif variable_1.public_interface == 'in' and variable_2.public_interface == 'out':
-                return variable_2.name, variable_1.name
+            # Validation makese sure the public_interface are in/out
+            # and if they are equal it would trigger an error at the connection stage (Target already assigned)
+            if variable_1.public_interface == 'out':
+                return variable_1, variable_2
+            else:
+                return variable_2, variable_1
         else:
             # determine which component is parent of the other
             if _parent_of(comp_1, comp_2):
@@ -482,10 +515,9 @@ class Parser(object):
 
             # parent/child components are connected using private/public interface, respectively
             if child_var.public_interface == 'in' and parent_var.private_interface == 'out':
-                return parent_var.name, child_var.name
+                return parent_var, child_var
             elif child_var.public_interface == 'out' and parent_var.private_interface == 'in':
-                return child_var.name, parent_var.name
-
+                return child_var, parent_var
         raise ValueError('Cannot determine the source & target for connection (%s, %s) - (%s, %s)' %
                          (comp_1, var_1, comp_2, var_2))
 
@@ -499,8 +531,9 @@ class Parser(object):
         Non state variables with an initial value are treated as constants. For consistent processing later on we add
         equations defining them, and remove the initial_value attribute.
         """
-        for var in self.model.variables():
-            if var in self.model.get_state_variables():
+        state_vars = set(self.model.get_state_variables())
+        for var in set(self.model.variables()):
+            if var in state_vars:
                 assert var.initial_value is not None, 'State variable {} has no initial_value set'.format(var)
             elif var.initial_value is not None:
                 value = self.model.create_quantity(var.initial_value, var.units)
